@@ -10,12 +10,46 @@
  * Implementation of CodePlug::Flags
  * ********************************************************************************************* */
 Codeplug::Flags::Flags()
-  : updateCodePlug(true), autoEnableGPS(false), autoEnableRoaming(false)
+  : TransferFlags(), _updateCodeplug(true), _autoEnableGPS(false), _autoEnableRoaming(false)
 {
   // pass...
 }
 
 
+bool
+Codeplug::Flags::updateCodeplug() const {
+  return _updateCodeplug;
+}
+
+void
+Codeplug::Flags::setUpdateCodeplug(bool enable) {
+  _updateCodeplug = enable;
+}
+
+
+bool
+Codeplug::Flags::autoEnableGPS() const {
+  return _autoEnableGPS;
+}
+
+void
+Codeplug::Flags::setAutoEnableGPS(bool enable) {
+  _autoEnableGPS = enable;
+}
+
+
+bool
+Codeplug::Flags::autoEnableRoaming() const {
+  return _autoEnableRoaming;
+}
+
+void
+Codeplug::Flags::setAutoEnableRoaming(bool enable) {
+  _autoEnableRoaming = enable;
+}
+
+
+
 /* ********************************************************************************************* *
  * Implementation of CodePlug::Element
  * ********************************************************************************************* */
@@ -670,6 +704,9 @@ Codeplug::Context::Context(Config *config)
   addTable(&RoamingChannel::staticMetaObject);
   addTable(&RoamingZone::staticMetaObject);
   addTable(&SMSTemplate::staticMetaObject);
+  addTable(&BasicEncryptionKey::staticMetaObject);
+  addTable(&ARC4EncryptionKey::staticMetaObject);
+  addTable(&AESEncryptionKey::staticMetaObject);
 }
 
 Config  *
