@@ -252,8 +252,10 @@ void
 Config::clear() {
   ConfigItem::clear();
 
-  // Reset lists
+  // Reset settings
   _settings->clear();
+
+  // Reset lists
   _radioIDs->clear();
   _contacts->clear();
   _rxGroupLists->clear();
@@ -264,6 +266,11 @@ Config::clear() {
   _roamingChannels->clear();
   _roamingZones->clear();
 
+  // Clear extensions
+  commercialExtension()->clear();
+  smsExtension()->clear();
+  setTyTExtension(nullptr);
+
   emit modified(this);
 }
 
@@ -325,6 +332,7 @@ Config::readCSV(QTextStream &stream, QString &errorMessage)
   return true;
 }
 
+
 bool
 Config::readYAML(const QString &filename, const ErrorStack &err) {
   YAML::Node node;
@@ -357,9 +365,11 @@ Config::readYAML(const QString &filename, const ErrorStack &err) {
   if (! link(node, context, err))
     return false;
 
+  setModified(false);
   return true;
 }
 
+
 bool
 Config::parse(const YAML::Node &node, Context &ctx, const ErrorStack &err)
 {
