@@ -7,6 +7,7 @@
 #include "radioid.hh"
 #include "rxgrouplist.hh"
 #include "roamingzone.hh"
+#include "zone.hh"
 #include "encryptionextension.hh"
 
 
@@ -59,7 +60,8 @@ ConfigObjectReference::set(ConfigObject *object) {
 
   _object = object;
   if (_object)
-    connect(_object, SIGNAL(destroyed(QObject*)), this, SLOT(onReferenceDeleted(QObject*)));
+    connect(_object, &QObject::destroyed,
+            this, &ConfigObjectReference::onReferenceDeleted, Qt::DirectConnection);
 
   emit modified();
   return true;
@@ -299,3 +301,15 @@ EncryptionKeyReference::EncryptionKeyReference(QObject *parent)
 {
   // pass...
 }
+
+
+/* ********************************************************************************************* *
+ * Implementation of ZoneReference
+ * ********************************************************************************************* */
+ZoneReference::ZoneReference(QObject *parent)
+  : ConfigObjectReference(Zone::staticMetaObject, parent)
+{
+  // pass...
+}
+
+
