@@ -2,7 +2,6 @@
 #include "config.hh"
 #include "utils.hh"
 #include "logger.hh"
-#include "opengd77_extension.hh"
 
 
 /* ********************************************************************************************* *
@@ -263,10 +262,10 @@ ContactList::ContactList(QObject *parent)
 }
 
 int
-ContactList::add(ConfigObject *obj, int row) {
+ContactList::add(ConfigObject *obj, int row, bool unique) {
   if ((nullptr == obj) || (! obj->is<Contact>()))
     return -1;
-  return ConfigObjectList::add(obj, row);
+  return ConfigObjectList::add(obj, row, unique);
 }
 
 int
