@@ -55,12 +55,12 @@ D868UVCallsignDB::EntryElement::setContent(
     const QString &country, const QString &comment)
 {
   unsigned addr = 0x0006;
-  writeASCII(addr, name, 16, 0x00); addr += std::min(16, name.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, city, 15, 0x00); addr += std::min(15, city.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, call, 8, 0x00); addr += std::min(8, call.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, state, 16, 0x00); addr += std::min(16, state.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, country, 16, 0x00); addr += std::min(16, country.size()); setUInt8(addr, 0); addr++;
-  writeASCII(addr, comment, 16, 0x00); addr += std::min(16, comment.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, name, 16, 0x00); addr += std::min(qsizetype(16), name.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, city, 15, 0x00); addr += std::min(qsizetype(15), city.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, call, 8, 0x00); addr += std::min(qsizetype(8), call.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, state, 16, 0x00); addr += std::min(qsizetype(16), state.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, country, 16, 0x00); addr += std::min(qsizetype(16), country.size()); setUInt8(addr, 0); addr++;
+  writeASCII(addr, comment, 16, 0x00); addr += std::min(qsizetype(16), comment.size()); setUInt8(addr, 0); addr++;
 }
 
 unsigned
@@ -76,11 +76,11 @@ D868UVCallsignDB::EntryElement::fromUser(const UserDatabase::User &user) {
 unsigned
 D868UVCallsignDB::EntryElement::size(const UserDatabase::User &user) {
   return 6 // header
-      + std::min(16, user.name.size())+1 // name
-      + std::min(15, user.city.size())+1 // city
-      + std::min( 8, user.call.size())+1 // call
-      + std::min(16, user.state.size())+1 // state
-      + std::min(16, user.country.size())+1 // country
+      + std::min(qsizetype(16), user.name.size())+1 // name
+      + std::min(qsizetype(15), user.city.size())+1 // city
+      + std::min(qsizetype( 8), user.call.size())+1 // call
+      + std::min(qsizetype(16), user.state.size())+1 // state
+      + std::min(qsizetype(16), user.country.size())+1 // country
       + 1; // no comment but 0x00 terminator
 }
 
@@ -145,7 +145,7 @@ D868UVCallsignDB::D868UVCallsignDB(QObject *parent)
   addImage("AnyTone AT-D878UV Callsign database.");
 }
 
-bool D868UVCallsignDB::encode(UserDatabase *db, const Selection &selection, const ErrorStack &err) {
+bool D868UVCallsignDB::encode(UserDatabase *db, const Flags &selection, const ErrorStack &err) {
   Q_UNUSED(err)
 
   // Determine size of call-sign DB in memory
