@@ -19,6 +19,11 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
   _callSignDBImplemented  = true;
   _numCallSignDBEntries   = 200000;
 
+  // Define limits for satellite config
+  _hasSatelliteConfig          = false;
+  _satelliteConfigImplemented  = false;
+  _numSatellites               = 0;
+
   /* Define limits for the general settings. */
   add("settings",
       new RadioLimitItem{
@@ -37,7 +42,7 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
       new RadioLimitList {
         { DMRRadioID::staticMetaObject, 1, 250, new RadioLimitObject {
             {"name", new RadioLimitString(1,8, RadioLimitString::ASCII) },
-            {"id", new RadioLimitUInt(0, 16777215)}
+            {"number", new RadioLimitDMRId(RadioLimitIssue::Severity::Critical)}
           } }
       });
 
@@ -52,7 +57,7 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
                 (unsigned)DMRContact::GroupCall,
                 (unsigned)DMRContact::AllCall
               }},
-            { "number", new RadioLimitUInt(0, 16777215) }
+            { "number", new RadioLimitDMRId(RadioLimitIssue::Severity::Hint) }
           } },
         { DTMFContact::staticMetaObject, 0, 128, new RadioLimitObject {
             { "name", new RadioLimitString(1, 15, RadioLimitString::ASCII) },
@@ -76,7 +81,7 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
           { FMChannel::staticMetaObject,
             new RadioLimitObject {
               {"name", new RadioLimitString(1, 16, RadioLimitString::ASCII)},
-              {"rxFrequency", new RadioLimitFrequencies(rxFreqRanges, true)},
+              {"rxFrequency", new RadioLimitFrequencies(rxFreqRanges, RadioLimitIssue::Severity::Critical)},
               {"txFrequency", new RadioLimitTransmitFrequencies(txFreqRanges)},
               {"power", new RadioLimitEnum{unsigned(Channel::Power::Low), unsigned(Channel::Power::High)}},
               {"timeout", new RadioLimitUInt(0, -1, std::numeric_limits<unsigned>::max())},
@@ -100,7 +105,7 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
           { DMRChannel::staticMetaObject,
             new RadioLimitObject {
               {"name", new RadioLimitString(1,16, RadioLimitString::ASCII)},
-              {"rxFrequency", new RadioLimitFrequencies(rxFreqRanges, true)},
+              {"rxFrequency", new RadioLimitFrequencies(rxFreqRanges, RadioLimitIssue::Severity::Critical)},
               {"txFrequency", new RadioLimitTransmitFrequencies(txFreqRanges)},
               {"power", new RadioLimitEnum{unsigned(Channel::Power::Low), unsigned(Channel::Power::High)}},
               {"timeout", new RadioLimitUInt(0, -1, std::numeric_limits<unsigned>::max())},
@@ -138,7 +143,7 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
   /* Define limits for scan lists. */
   add("scanlists",
       new RadioLimitList(
-        ScanList::staticMetaObject, 1, 250, new RadioLimitObject{
+        ScanList::staticMetaObject, 0, 250, new RadioLimitObject{
           { "name", new RadioLimitString(1, 16, RadioLimitString::ASCII) },
           { "primary", new RadioLimitObjRef(Channel::staticMetaObject, false) },
           { "secondary", new RadioLimitObjRef(Channel::staticMetaObject, true) },
@@ -147,13 +152,13 @@ D868UVLimits::D868UVLimits(const std::initializer_list<std::pair<Frequency, Freq
         }));
 
   /* Ignore positioning systems. */
-  add("positioning", new RadioLimitList(
-        GPSSystem::staticMetaObject, 0, 1, new RadioLimitObject {
+  add("positioning", new RadioLimitList({
+        { GPSSystem::staticMetaObject, 0, 1, new RadioLimitObject {
           { "name", new RadioLimitStringIgnored() },
           { "period", new RadioLimitUInt(0, 7650) },
           { "contact", new RadioLimitObjRef(DMRContact::staticMetaObject, false) },
-          { "revert", new RadioLimitObjRef(DMRChannel::staticMetaObject, true) }
-        } ) );
+          { "revert", new RadioLimitObjRef({SelectedChannel::staticMetaObject, DMRChannel::staticMetaObject}, true) } } },
+        { APRSSystem::staticMetaObject, 0, -1, new RadioLimitIgnored() } } ) );
 
 
   /* Ignore roaming zones. */
