@@ -18,6 +18,7 @@ D878UV::D878UV(AnytoneInterface *device, QObject *parent)
   _codeplug = new D878UVCodeplug(this);
   _codeplug->clear();
   _callsigns = new D868UVCallsignDB(this);
+  _satellites = new AnytoneSatelliteConfig(this);
 
   // Get device info and determine supported TX frequency bands
   AnytoneInterface::RadioVariant info;
@@ -145,5 +146,5 @@ D878UV::limits() const {
 RadioInfo
 D878UV::defaultRadioInfo() {
   return RadioInfo(
-        RadioInfo::D878UV, "d878uv", "AT-D878UV", "AnyTone", AnytoneInterface::interfaceInfo());
+        RadioInfo::D878UV, "d878uv", "AT-D878UV", "AnyTone", {AnytoneGD32Interface::interfaceInfo()});
 }
