@@ -1,5 +1,6 @@
 #include "dm1701_codeplug.hh"
 #include "logger.hh"
+#include "config.hh"
 #include <QTimeZone>
 
 
@@ -38,10 +39,6 @@
 #define ADDR_GPSSYSTEMS         0x03ec40
 #define GPSSYSTEM_SIZE          0x000010
 
-#define NUM_TEXTMESSAGES              50
-#define ADDR_TEXTMESSAGES       0x002180
-#define TEXTMESSAGE_SIZE        0x000120
-
 #define ADDR_EMERGENCY_SETTINGS 0x005a50
 #define NUM_EMERGENCY_SYSTEMS         32
 #define ADDR_EMERGENCY_SYSTEMS  0x005a60
@@ -118,8 +115,8 @@ DM1701Codeplug::ChannelElement::setPower(Channel::Power pwr) {
 }
 
 Channel *
-DM1701Codeplug::ChannelElement::toChannelObj() const {
-  Channel *ch = TyTCodeplug::ChannelElement::toChannelObj();
+DM1701Codeplug::ChannelElement::toChannelObj(const ErrorStack &err) const {
+  Channel *ch = TyTCodeplug::ChannelElement::toChannelObj(err);
   if (nullptr == ch)
     return ch;
 
@@ -130,6 +127,7 @@ DM1701Codeplug::ChannelElement::toChannelObj() const {
     ch->tytChannelExtension()->enableTightSquelch(tightSquelchEnabled());
     ch->tytChannelExtension()->enableReverseBurst(reverseBurst());
   }
+
   return ch;
 }
 
@@ -876,9 +874,20 @@ DM1701Codeplug::decodePrivacyKeys(Config *config, Context &ctx, const ErrorStack
 
 void
 DM1701Codeplug::clearTextMessages() {
-  memset(data(ADDR_TEXTMESSAGES), 0, NUM_TEXTMESSAGES*TEXTMESSAGE_SIZE);
+  MessageBankElement(data(Offset::messages())).clear();
 }
 
+bool
+DM1701Codeplug::encodeTextMessages(Context &ctx, const Flags &flags, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).encode(ctx, flags, err);
+}
+
+bool
+DM1701Codeplug::decodeTextMessages(Context &ctx, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).decode(ctx, err);
+}
+
+
 void
 DM1701Codeplug::clearEmergencySystems() {
   EmergencySettingsElement(data(ADDR_EMERGENCY_SETTINGS)).clear();
