@@ -229,7 +229,8 @@ GD73Codeplug::SettingsElement::squelch() const {
 }
 void
 GD73Codeplug::SettingsElement::setSquelch(unsigned int level) {
-  setUInt8(Offset::squelchLevel(), level*9/10);
+  level = std::min(9U, level);
+  setUInt8(Offset::squelchLevel(), level);
 }
 
 bool
@@ -1662,6 +1663,7 @@ GD73Codeplug::ChannelElement::toChannel(Context &ctx, const ErrorStack &err) {
   ch->setRXFrequency(rxFrequency());
   ch->setTXFrequency(txFrequency());
   ch->setRXOnly(rxOnly());
+  ch->setPower(power());
 
   return ch;
 }
@@ -1725,6 +1727,7 @@ GD73Codeplug::ChannelElement::encode(Channel *ch, Context &ctx, const ErrorStack
   enableRXOnly(ch->rxOnly());
   if (! ch->scanListRef()->isNull())
     setScanListIndex(ctx.index(ch->scanList()));
+  setPower(ch->power());
 
   // Dispatch by type
   if (ch->is<DMRChannel>()) {
@@ -1748,6 +1751,7 @@ GD73Codeplug::ChannelElement::encode(Channel *ch, Context &ctx, const ErrorStack
         setEncryptionKeyIndex(ctx.index(ext->encryptionKey()));
   } else if (ch->is<FMChannel>()) {
     FMChannel *fm = ch->as<FMChannel>();
+    setType(ChannelElement::Type::FM);
     switch (fm->admit()) {
     case FMChannel::Admit::Always: setAdmit(Admit::Always); break;
     case FMChannel::Admit::Tone: setAdmit(Admit::CC_CTCSS); break;
@@ -2318,7 +2322,6 @@ GD73Codeplug::encode(Config *config, const Flags &flags, const ErrorStack &err)
   Q_UNUSED(flags);
 
   Context ctx(config);
-  ctx.addTable(&BasicEncryptionKey::staticMetaObject);
 
   if (! index(config, ctx, err)) {
     errMsg(err) << "Cannot encode codeplug for Radioddity GD73.";
@@ -2376,7 +2379,6 @@ GD73Codeplug::encode(Config *config, const Flags &flags, const ErrorStack &err)
 bool
 GD73Codeplug::decode(Config *config, const ErrorStack &err) {
   Context ctx(config);
-  ctx.addTable(&BasicEncryptionKey::staticMetaObject);
 
   if (! decodeTimestamp(ctx, err)) {
     errMsg(err) << "Cannot decode codeplug.";
