@@ -42,5 +42,5 @@ OpenGD77::callsignDB() {
 RadioInfo
 OpenGD77::defaultRadioInfo() {
   return RadioInfo(
-        RadioInfo::OpenGD77, "opengd77", "OpenGD77", "OpenGD77 Project", OpenGD77Interface::interfaceInfo());
+        RadioInfo::OpenGD77, "opengd77", "OpenGD77", "OpenGD77 Project", {OpenGD77Interface::interfaceInfo()});
 }
