@@ -17,7 +17,7 @@ OpenGD77Codeplug::OpenGD77Codeplug(QObject *parent)
   image(EEPROM).addElement(0x07500, 0x03b00);
 
   addImage("OpenGD77 Codeplug FLASH");
-  image(FLASH).addElement(0x00000, 0x011a0);
+  image(FLASH).addElement(0x00000, AdditionalSettingsElement::size());
   image(FLASH).addElement(0x7b000, 0x13e60);
 }
 
@@ -29,7 +29,7 @@ OpenGD77Codeplug::clearGeneralSettings() {
 bool
 OpenGD77Codeplug::encodeGeneralSettings(const Flags &flags, Context &ctx, const ErrorStack &err) {
   GeneralSettingsElement el(data(Offset::settings(), ImageIndex::settings()));
-  if (! flags.updateCodePlug)
+  if (! flags.updateCodeplug())
     el.clear();
 
   return el.encode(ctx, err);
@@ -67,7 +67,7 @@ OpenGD77Codeplug::clearAPRSSettings() {
 bool
 OpenGD77Codeplug::encodeAPRSSettings(const Flags &flags, Context &ctx, const ErrorStack &err) {
   APRSSettingsBankElement el(data(Offset::aprsSettings(), ImageIndex::aprsSettings()));
-  if (! flags.updateCodePlug)
+  if (! flags.updateCodeplug())
     el.clear();
 
   return el.encode(ctx, err);
@@ -79,6 +79,12 @@ OpenGD77Codeplug::decodeAPRSSettings(Context &ctx, const ErrorStack &err) {
       .decode(ctx, err);
 }
 
+bool
+OpenGD77Codeplug::linkAPRSSettings(Context &ctx, const ErrorStack &err) {
+  return APRSSettingsBankElement(data(Offset::aprsSettings(), ImageIndex::aprsSettings()))
+      .link(ctx, err);
+}
+
 
 bool
 OpenGD77Codeplug::encodeBootSettings(const Flags &flags, Context &ctx, const ErrorStack &err) {
@@ -179,7 +185,7 @@ OpenGD77Codeplug::createChannels(Context &ctx, const ErrorStack &err) {
     else
       bank = ChannelBankElement(data(Offset::channelBank1() + (b-1)*ChannelBankElement::size(), ImageIndex::channelBank1()));
 
-    for (unsigned int i=0; i < ChannelBankElement::Limit::channelCount(); i++, c++) {
+    for (unsigned int i=0; i < ChannelBankElement::Limit::channelCount(); i++) {
       if (! bank.isEnabled(i))
         continue;
 
@@ -189,7 +195,7 @@ OpenGD77Codeplug::createChannels(Context &ctx, const ErrorStack &err) {
         return false;
       }
       ctx.config()->channelList()->add(obj);
-      ctx.add(obj, c);
+      ctx.add(obj, c++);
     }
   }
 
@@ -205,7 +211,7 @@ OpenGD77Codeplug::linkChannels(Context &ctx, const ErrorStack &err) {
     else
       bank = ChannelBankElement(data(Offset::channelBank1() + (b-1)*ChannelBankElement::size(), ImageIndex::channelBank1()));
 
-    for (unsigned int i=0; i < ChannelBankElement::Limit::channelCount(); i++, c++) {
+    for (unsigned int i=0; i < ChannelBankElement::Limit::channelCount(); i++) {
       if (! bank.isEnabled(i))
         continue;
 
@@ -215,6 +221,7 @@ OpenGD77Codeplug::linkChannels(Context &ctx, const ErrorStack &err) {
                     << "' from index " << i << " in bank " << b << ".";
         return false;
       }
+      c++;
     }
   }
 
