@@ -2,6 +2,9 @@
 #include "logger.hh"
 #include "radioinfo.hh"
 #include <QtEndian>
+#include <QDateTime>
+#include <QTimeZone>
+
 
 #define USB_VID 0x1fc9
 #define USB_PID 0x0094
@@ -164,6 +167,13 @@ OpenGD77Interface::CommandRequest::initCommand(Option option) {
   memset(this->message, 0, sizeof(this->message));
 }
 
+void
+OpenGD77Interface::CommandRequest::initSetDateTime(const QDateTime &dt) {
+  initCommand(SET_DATETIME);
+  this->timestamp = qToLittleEndian(dt.toUTC().toSecsSinceEpoch());
+}
+
+
 
 /* ********************************************************************************************* *
  * Implementation of OpenGD77Interface
@@ -221,16 +231,10 @@ OpenGD77Interface::identifier(const ErrorStack &err) {
   case FirmwareInfo::RadioType::MD380:
   case FirmwareInfo::RadioType::DM1701:
   case FirmwareInfo::RadioType::DM1701RGB:
+  case FirmwareInfo::RadioType::MD9600:
+  case FirmwareInfo::RadioType::MD2017:
     _protocolVariant = Variant::UV380;
   return RadioInfo::byID(RadioInfo::OpenUV380);
-
-  case FirmwareInfo::RadioType::MD9600:
-    logInfo() << "OpenGD77 variant MD9600 not supported (yet).";
-  return RadioInfo();
-
-  case FirmwareInfo::RadioType::MD2017:
-    logInfo() << "OpenGD77 variant MD2017 not supported (yet).";
-  return RadioInfo();
   }
 
   errMsg(err) << "Unknown OpenGD77 variant " << info.radioType << ".";
@@ -391,6 +395,11 @@ OpenGD77Interface::read_finish(const ErrorStack &err) {
   return true;
 }
 
+bool
+OpenGD77Interface::setDateTime(const QDateTime &datetime, const ErrorStack &err) {
+  return sendSetDateTime(datetime, err);
+}
+
 bool
 OpenGD77Interface::reboot(const ErrorStack &err) {
   return sendCommand(CommandRequest::SAVE_SETTINGS_NOT_VFOS, err);
@@ -834,6 +843,39 @@ OpenGD77Interface::sendCloseScreen(const ErrorStack &err) {
   return true;
 }
 
+
+bool
+OpenGD77Interface::sendSetDateTime(const QDateTime &dt, const ErrorStack &err) {
+  CommandRequest req; req.initSetDateTime(dt);
+  uint8_t resp;
+
+  if (sizeof(CommandRequest) != QSerialPort::write((const char *) &req, sizeof(CommandRequest))) {
+    errMsg(err) << "Cannot write to serial port.";
+    return false;
+  }
+
+  if (! waitForReadyRead(1000)) {
+    errMsg(err) << "Cannot read from serial port: Timeout!";
+    return false;
+  }
+
+  int retlen = QSerialPort::read((char *)&resp, 1);
+
+  if (0 > retlen) {
+    errMsg(err) << "Cannot read from serial port.";
+    return false;
+  } else if (0 == retlen) {
+    errMsg(err) << "Cannot send command: Device returned empty message.";
+    return false;
+  } else if ('-' != resp) {
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
+    return false;
+  }
+
+  return true;
+}
+
+
 bool
 OpenGD77Interface::sendCommand(CommandRequest::Option option, const ErrorStack &err) {
   CommandRequest req; req.initCommand(option);
