@@ -6,6 +6,8 @@
 #include "zone.hh"
 #include "scanlist.hh"
 #include "roamingzone.hh"
+#include "opengd77_satelliteconfig.hh"
+
 
 
 OpenGD77Limits::OpenGD77Limits(QObject *parent)
@@ -16,6 +18,11 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
   _callSignDBImplemented  = true;
   _numCallSignDBEntries   = 15796;
 
+  // Define limits for satellite config
+  _hasSatelliteConfig          = true;
+  _satelliteConfigImplemented  = true;
+  _numSatellites               = OpenGD77SatelliteConfig::Limit::satellites();
+
   /* Define limits for the general settings. */
   add("settings",
       new RadioLimitItem{
@@ -34,7 +41,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
       new RadioLimitList {
         { DMRRadioID::staticMetaObject, 1, 1, new RadioLimitObject {
             {"name", new RadioLimitString(1,8, RadioLimitString::ASCII) },
-            {"id", new RadioLimitUInt(0, 16777215)}
+            {"number", new RadioLimitDMRId(RadioLimitIssue::Severity::Critical)}
           } }
         /// @todo check default radio ID.
       });
@@ -50,7 +57,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
                 (unsigned)DMRContact::GroupCall,
                 (unsigned)DMRContact::AllCall
               }},
-            { "number", new RadioLimitUInt(0, 16777215) }
+            { "number", new RadioLimitDMRId(RadioLimitIssue::Severity::Hint) }
           } },
         { DTMFContact::staticMetaObject, 0, 32, new RadioLimitObject {
             { "name", new RadioLimitString(1, 16, RadioLimitString::ASCII) },
@@ -62,7 +69,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
   /* Define limits for group lists. */
   add("groupLists",
       new RadioLimitList(
-            RXGroupList::staticMetaObject, 1, 76, new RadioLimitObject {
+            RXGroupList::staticMetaObject, 0, 76, new RadioLimitObject {
               { "name", new RadioLimitString(1, 16, RadioLimitString::ASCII) },
               // allow for any digital contact reference
               { "contacts", new RadioLimitRefList(1, 32, DMRContact::staticMetaObject) }
@@ -77,7 +84,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
             new RadioLimitObject {
               {"name", new RadioLimitString(1, 16, RadioLimitString::ASCII)},
               {"rxFrequency", new RadioLimitFrequencies({{Frequency::fromMHz(136.), Frequency::fromMHz(174.)},
-                                                         {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}}, true)},
+                                                         {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}}, RadioLimitIssue::Severity::Critical)},
               {"txFrequency", new RadioLimitTransmitFrequencies({{Frequency::fromMHz(136.), Frequency::fromMHz(174.)},
                                                                  {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}})},
               {"power", new RadioLimitEnum{unsigned(Channel::Power::Low), unsigned(Channel::Power::High)}},
@@ -104,7 +111,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
             new RadioLimitObject {
               {"name", new RadioLimitString(1,16, RadioLimitString::ASCII)},
               {"rxFrequency", new RadioLimitFrequencies({{Frequency::fromMHz(136.), Frequency::fromMHz(174.)},
-                                                         {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}}, true)},
+                                                         {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}}, RadioLimitIssue::Severity::Critical)},
               {"txFrequency", new RadioLimitTransmitFrequencies({{Frequency::fromMHz(136.), Frequency::fromMHz(174.)},
                                                                  {Frequency::fromMHz(400.), Frequency::fromMHz(470.)}})},
               {"power", new RadioLimitEnum{unsigned(Channel::Power::Low), unsigned(Channel::Power::High)}},
@@ -134,7 +141,7 @@ OpenGD77Limits::OpenGD77Limits(QObject *parent)
   /* Define limits for zone list. */
   add("zones",
       new RadioLimitList(
-        Zone::staticMetaObject, 1, 68, new RadioLimitSingleZone(
+        Zone::staticMetaObject, 0, 68, new RadioLimitSingleZone(
           80, {
             { "name", new RadioLimitString(1, 16, RadioLimitString::ASCII) }, // 16 ASCII chars in name
             { "anytone", new RadioLimitIgnored(RadioLimitIssue::Hint) }     // ignore AnyTone extensions
