@@ -1,9 +1,11 @@
 #include "opengd77base_codeplug.hh"
+#include "opengd77_extension.hh"
 #include "radioid.hh"
 #include "config.hh"
 #include "logger.hh"
 #include "intermediaterepresentation.hh"
 #include "satellitedatabase.hh"
+#include <QRegularExpression>
 
 
 /* ********************************************************************************************* *
@@ -14,13 +16,13 @@ OpenGD77BaseCodeplug::encodeAngle(double angle) {
   uint32_t sign = (angle < 0) ? 1 : 0;
   uint32_t decimals = std::abs(int(angle * 10000));
   uint32_t deg  = decimals/10000; decimals = decimals % 10000;
-  return (sign << 23) | (deg <<15) | decimals;
+  return (sign << 23) | (deg << 15) | decimals;
 }
 
 double
 OpenGD77BaseCodeplug::decodeAngle(uint32_t code) {
   return (((code >> 23) & 1) ? -1 : 1) * (
-        ((code >> 15) & 0xff) + double(code & 0x7ff)/10000
+        ((code >> 15) & 0xff) + double(code & 0x7fff)/10000
         );
 }
 
@@ -40,7 +42,11 @@ OpenGD77BaseCodeplug::encodeSelectiveCall(const SelectiveCall &call) {
     toneCode = call.mHz()/100;
   }
 
-  return (dcs<<15) | (inverted << 14) | (toneCode & 0x3fff);
+  uint16_t bcd = (((toneCode/1000) % 10 ) << 12) |
+      (((toneCode/100) % 10 ) << 8) |
+      (((toneCode/10) % 10 ) << 4) |
+      (((toneCode/1) % 10 )  << 0);
+  return (dcs<<15) | (inverted << 14) | (bcd & 0x3fff);
 }
 
 SelectiveCall
@@ -50,7 +56,12 @@ OpenGD77BaseCodeplug::decodeSelectiveCall(uint16_t code) {
 
   bool dcs = ((code >> 15) & 1),
       inverted = ((code >> 14) & 1);
-  code &= 0x3fff;
+
+  uint16_t bcd = (code & 0x3fff);
+  code = 1000 * ((bcd >> 12) & 0xf)
+      + 100 * ((bcd >> 8) & 0xf)
+      + 10 * ((bcd >> 4) & 0xf)
+      + 1 * ((bcd >> 0) & 0xf);
 
   if (! dcs)
     return SelectiveCall(double(code)/10);
@@ -93,7 +104,7 @@ OpenGD77BaseCodeplug::ChannelElement::clear() {
   enablePowerSave(false);
   enableBeep(false);
   clearDMRId();
-  setGroupListIndex(0);
+  clearGroupListIndex();
   setColorCode(0);
   clearAPRSIndex();
   clearTXContact();
@@ -195,6 +206,25 @@ OpenGD77BaseCodeplug::ChannelElement::clearPower() {
 }
 
 
+Interval
+OpenGD77BaseCodeplug::ChannelElement::transmitTimeout() const {
+  if (0 == getUInt8(Offset::txTimeout()))
+    return Interval::infinity();
+  return Interval::fromSeconds((unsigned)getUInt8(Offset::txTimeout())*15);
+}
+
+void
+OpenGD77BaseCodeplug::ChannelElement::setTransmitTimeout(const Interval &interval) {
+  if (interval.isInfinite())
+    setUInt8(Offset::txTimeout(), 0);
+  else {
+    unsigned s = interval.seconds()/15;
+    s = std::max(1u, std::min(33u, s));
+    setUInt8(Offset::txTimeout(), s);
+  }
+}
+
+
 bool
 OpenGD77BaseCodeplug::ChannelElement::hasFixedPosition() const {
   return getBit(Offset::useFixedLocation());
@@ -202,11 +232,11 @@ OpenGD77BaseCodeplug::ChannelElement::hasFixedPosition() const {
 
 QGeoCoordinate
 OpenGD77BaseCodeplug::ChannelElement::fixedPosition() const {
-  uint32_t latCode = (((uint32_t)getUInt8(Offset::latitude2())) << 24) +
-      (((uint32_t)getUInt8(Offset::latitude1())) << 16) +
+  uint32_t latCode = (((uint32_t)getUInt8(Offset::latitude2())) << 16) +
+      (((uint32_t)getUInt8(Offset::latitude1())) << 8) +
       ((uint32_t)getUInt8(Offset::latitude0()));
-  uint32_t lonCode = (((uint32_t)getUInt8(Offset::longitude2())) << 24) +
-      (((uint32_t)getUInt8(Offset::longitude1())) << 16) +
+  uint32_t lonCode = (((uint32_t)getUInt8(Offset::longitude2())) << 16) +
+      (((uint32_t)getUInt8(Offset::longitude1())) << 8) +
       ((uint32_t)getUInt8(Offset::longitude0()));
 
   return QGeoCoordinate(decodeAngle(latCode), decodeAngle(lonCode));
@@ -214,7 +244,7 @@ OpenGD77BaseCodeplug::ChannelElement::fixedPosition() const {
 
 void
 OpenGD77BaseCodeplug::ChannelElement::setFixedPosition(const QGeoCoordinate &coordinate) {
-  if (!coordinate.isValid()) {
+  if (! coordinate.isValid()) {
     clearFixedPosition();
     return;
   }
@@ -537,26 +567,38 @@ OpenGD77BaseCodeplug::ChannelElement::decode(Codeplug::Context &ctx, const Error
   // Apply common settings
   ch->setName(name());
   ch->setRXFrequency(rxFrequency());
+
   if (isSimplex())
     ch->setTXFrequency(rxFrequency());
   else
     ch->setTXFrequency(txFrequency());
+
   if (globalPower())
     ch->setDefaultPower();
   else
     ch->setPower(power());
+
   ch->setRXOnly(rxOnly());
+
   if (vox())
     ch->setVOXDefault();
   else
     ch->disableVOX();
 
+  if (transmitTimeout().isInfinite())
+    ch->disableTimeout();
+  else
+    ch->setTimeout(transmitTimeout().seconds());
+
   ch->setOpenGD77ChannelExtension(new OpenGD77ChannelExtension());
   ch->openGD77ChannelExtension()->enableScanZoneSkip(skipZoneScan());
   ch->openGD77ChannelExtension()->enableScanAllSkip(skipScan());
   ch->openGD77ChannelExtension()->enableBeep(beep());
   ch->openGD77ChannelExtension()->enablePowerSave(powerSave());
-  ch->openGD77ChannelExtension()->setLocation(fixedPosition());
+  if (hasFixedPosition())
+    ch->openGD77ChannelExtension()->setLocation(fixedPosition());
+  else
+    ch->openGD77ChannelExtension()->setLocation(QGeoCoordinate());
   ch->openGD77ChannelExtension()->setTalkerAliasTS1(aliasTimeSlot1());
   ch->openGD77ChannelExtension()->setTalkerAliasTS2(aliasTimeSlot2());
 
@@ -570,22 +612,28 @@ OpenGD77BaseCodeplug::ChannelElement::link(Channel *c, Context &ctx, const Error
   Q_UNUSED(err)
 
   // Link common
-  // Link digital channel
+
   if (c->is<DMRChannel>()) {
+    // Link DMR channel
     DMRChannel *dc = c->as<DMRChannel>();
     if (hasGroupList() && ctx.has<RXGroupList>(groupListIndex()))
       dc->setGroupListObj(ctx.get<RXGroupList>(groupListIndex()));
     if (hasTXContact() && ctx.has<DMRContact>(txContactIndex()))
       dc->setTXContactObj(ctx.get<DMRContact>(txContactIndex()));
-    if (hasDMRId()) {
+    // Testing dmrId() == 0 fixes a bug in the OpenGD77 firmware. May change in future.
+    if (hasDMRId() && (0 != dmrId())) {
+      logDebug() << "Channel '" << c->name() << "' overrides default DMR id with "
+                 << dmrId() << ".";
       auto id = ctx.config()->radioIDs()->find(dmrId());
       if (nullptr == id) {
+        logDebug() << "DMR Id " << dmrId() << " is not defined yet, create one as 'Unknown ID'.";
         id = new DMRRadioID(QString("Unknown ID"), dmrId());
         ctx.config()->radioIDs()->add(id);
       }
       dc->setRadioIdObj(id);
     }
   } else if (c->is<FMChannel>()) {
+    // Link FM channel
     auto fm = c->as<FMChannel>();
     if (hasAPRSIndex()) {
       if (! ctx.has<APRSSystem>(aprsIndex())) {
@@ -615,6 +663,10 @@ OpenGD77BaseCodeplug::ChannelElement::encode(const Channel *c, Context &ctx, con
     setPower(c->power());
 
   enableRXOnly(c->rxOnly());
+  if (c->timeoutDisabled())
+    setTransmitTimeout(Interval::infinity());
+  else
+    setTransmitTimeout(Interval::fromSeconds(c->timeout()));
 
   // Enable vox
   bool defaultVOXEnabled = (c->defaultVOX() && (!ctx.config()->settings()->voxDisabled()));
@@ -937,7 +989,13 @@ OpenGD77BaseCodeplug::APRSSettingsElement::APRSSettingsElement(uint8_t *ptr)
 void
 OpenGD77BaseCodeplug::APRSSettingsElement::clear() {
   Element::clear();
+
   setName("");
+  clearFixedPosition();
+  setUInt32_le(Offset::fmFrequency(), 0);
+
+  // Some random data, appears to be important
+  writeASCII(Offset::unknownBytes(), "RA", 2);
 }
 
 
@@ -1087,9 +1145,20 @@ OpenGD77BaseCodeplug::APRSSettingsElement::setBaudRate(BaudRate rate) {
 }
 
 
+Frequency
+OpenGD77BaseCodeplug::APRSSettingsElement::fmFrequency() const {
+  return Frequency::fromHz(getUInt32_le(Offset::fmFrequency())*10);
+}
+
+void
+OpenGD77BaseCodeplug::APRSSettingsElement::setFMFrequency(Frequency f) {
+  setUInt32_le(Offset::fmFrequency(), f.inHz()/10);
+}
+
 bool
 OpenGD77BaseCodeplug::APRSSettingsElement::encode(const APRSSystem *sys, const Context &ctx, const ErrorStack &err) {
   Q_UNUSED(ctx); Q_UNUSED(err);
+  clear();
 
   setName(sys->name());
   setSourceSSID(sys->srcSSID());
@@ -1097,13 +1166,14 @@ OpenGD77BaseCodeplug::APRSSettingsElement::encode(const APRSSystem *sys, const C
   QStringList vias = sys->path().split(",");
   unsigned int viaCount = 0;
   for (auto via: vias) {
-    QRegExp pattern("^([A-Z0-9]+)-(1?[0-9])$");
-    if (! pattern.exactMatch(via))
+    QRegularExpression pattern("^([A-Z0-9]+)-(1?[0-9])$");
+    auto match = pattern.match(via);
+    if (! match.hasMatch())
       continue;
     if (0 == viaCount)
-      setVia1(pattern.cap(1), pattern.cap(2).toUInt());
+      setVia1(match.captured(1), match.captured(2).toUInt());
     else if (1 == viaCount)
-      setVia2(pattern.cap(1), pattern.cap(2).toUInt());
+      setVia2(match.captured(1), match.captured(2).toUInt());
     else
       break;
     viaCount++;
@@ -1116,6 +1186,16 @@ OpenGD77BaseCodeplug::APRSSettingsElement::encode(const APRSSystem *sys, const C
   setBaudRate(BaudRate::Baud1200);
   setPositionPrecision(PositionPrecision::Max);
 
+  if(sys->hasRevertChannel()) {
+    setFMFrequency(sys->revertChannel()->txFrequency());
+  }
+
+  if (nullptr == sys->openGD77Extension())
+    return true;
+
+  if (sys->openGD77Extension()->location().isValid())
+    setFixedPosition(sys->openGD77Extension()->location());
+
   return true;
 }
 
@@ -1134,21 +1214,45 @@ OpenGD77BaseCodeplug::APRSSettingsElement::decode(const Context &ctx, const Erro
   sys->setDestination("APN000", 0);
   sys->setSrcSSID(sourceSSID());
   QStringList path;
-  if (hasVia1()) path.append(QString("%1-%2").arg(via1Call(), via1SSID()));
-  if (hasVia2()) path.append(QString("%1-%2").arg(via2Call(), via2SSID()));
+  if (hasVia1()) path.append(QString("%1-%2").arg(via1Call()).arg(via1SSID()));
+  if (hasVia2()) path.append(QString("%1-%2").arg(via2Call()).arg(via2SSID()));
   sys->setPath(path.join(","));
 
   sys->setIcon(icon());
   sys->setMessage(comment());
 
+  auto ext = new OpenGD77APRSSystemExtension();
+  sys->setOpenGD77Extension(ext);
+
+  if (hasFixedPosition())
+    ext->setLocation(fixedPosition());
+
   return sys;
 }
 
-
 bool
 OpenGD77BaseCodeplug::APRSSettingsElement::link(APRSSystem *sys, const Context &ctx, const ErrorStack &err) {
   Q_UNUSED(err);
 
+  if(fmFrequency().inHz() == 0) {
+    sys->resetRevertChannel();
+  } else {
+    // First, try to find a matching analog channel in list
+    FMChannel *ch = ctx.config()->channelList()->findFMChannelByTxFreq(fmFrequency());
+    if (! ch) {
+      // If no channel is found, create one with the settings from APRS channel:
+      ch = new FMChannel();
+      ch->setName("APRS Channel");
+      ch->setRXFrequency(fmFrequency());
+      ch->setTXFrequency(fmFrequency());
+      ch->setBandwidth(FMChannel::Bandwidth::Narrow);
+      logInfo() << "No matching APRS channel found for TX frequency " << double(fmFrequency().inHz())/1e6
+                << "MHz, create one as 'APRS Channel'";
+      ctx.config()->channelList()->add(ch);
+    }
+    sys->setRevertChannel(ch);
+  }
+
   if (ctx.config()->settings()->defaultId())
     sys->setSource(ctx.config()->settings()->defaultId()->name());
 
@@ -1221,6 +1325,24 @@ OpenGD77BaseCodeplug::APRSSettingsBankElement::decode(Context &ctx, const ErrorS
   return true;
 }
 
+bool
+OpenGD77BaseCodeplug::APRSSettingsBankElement::link(Context &ctx, const ErrorStack &err) {
+  for (unsigned int i=0; i<Limit::systems(); i++) {
+    if (system(i).isValid()) {
+      if (! ctx.has<APRSSystem>(i)) {
+        errMsg(err) << "Cannot link APRS system at index " << i << ": Not found in context.";
+        return false;
+      }
+      if (! system(i).link(ctx.get<APRSSystem>(i), ctx, err)) {
+        errMsg(err) << "Cannot link APRS system '" << ctx.get<APRSSystem>(i)->name()
+                    << "' at index " << i << ".";
+        return false;
+      }
+    }
+  }
+  return true;
+}
+
 
 
 /* ******************************************************************************************** *
@@ -1268,7 +1390,7 @@ OpenGD77BaseCodeplug::DTMFContactElement::number() const {
   QString number;
   uint8_t *ptr = _data + Offset::number();
   const QVector<char> lut = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','*','#'};
-  for (unsigned int i=0; (i<Limit::numberLength()) && (lut.size() < *ptr); i++, ptr++)
+  for (unsigned int i=0; (i<Limit::numberLength()) && (lut.size() > *ptr); i++, ptr++)
     number.append(lut[*ptr]);
   return number;
 }
@@ -1276,9 +1398,10 @@ OpenGD77BaseCodeplug::DTMFContactElement::number() const {
 void
 OpenGD77BaseCodeplug::DTMFContactElement::setNumber(const QString &number) {
   uint8_t *ptr = _data + Offset::number();
+  memset(ptr, 0xff, Limit::numberLength());
   unsigned int n = std::min(Limit::numberLength(), (unsigned int)number.length());
   const QVector<QChar> lut = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','*','#'};
-  for (unsigned int i=0; (i<n) && (lut.contains(number.at(i))); i++, ptr++)
+  for (unsigned int i=0; (i<n) && lut.contains(number.at(i)); i++, ptr++)
     *ptr = lut.indexOf(number.at(i));
 }
 
@@ -1690,272 +1813,6 @@ OpenGD77BaseCodeplug::ZoneBankElement::link(Context &ctx, const ErrorStack &err)
 
 
 
-/* ********************************************************************************************* *
- * Implementation of OpenGD77BaseCodeplug::OrbitalElement
- * ********************************************************************************************* */
-OpenGD77BaseCodeplug::OrbitalElement::OrbitalElement(uint8_t *ptr, size_t size)
-  : Element(ptr, size)
-{
-  // pass...
-}
-
-OpenGD77BaseCodeplug::OrbitalElement::OrbitalElement(uint8_t *ptr)
-  : Element(ptr, size())
-{
-  // pass...
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::clear() {
-  memset(_data, 0, size());
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::writeDigit(const Offset::Bit &offset, uint8_t digit) {
-  // Must be bit 0 or 4 (BCD)
-  if (offset.bit % 4)
-    return;
-
-  uint8_t val = getUInt8(offset.byte);
-  val &= ~(0xf << offset.bit);
-  val |= ((digit & 0xf) << offset.bit);
-  setUInt8(offset.byte, val);
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::writeInteger(const Offset::Bit &offset, int value, bool sign, unsigned int dec) {
-  unsigned int o = 0;
-  // Must be bit 0 or 4 (BCD)
-  if (offset.bit % 4)
-    return;
-  if (0 == dec)
-    return;
-
-  if (sign && 0 > value)
-    writeDigit(offset + o, 0xc); // '-' ?!?
-  else
-    writeDigit(offset + o, 0xb); // blank
-
-  o += 4*(dec-1);
-  for (int i=dec; i>0; i++, o = o - 4) {
-    if (value)
-      writeDigit(offset + o, value % 10);
-    else
-      writeDigit(offset + o, 0xb);
-    value /= 10;
-  }
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::writeFractional(const Offset::Bit &offset, double value, bool sign, unsigned int frac) {
-  unsigned int o = 0;
-  if (offset.bit % 4)
-    return;
-
-  if (0 == frac)
-    return;
-
-  if (sign && (0 > value))
-    writeDigit(offset + o, 0xc);
-  else
-    writeDigit(offset + o, 0xb);
-  o += 4;
-
-  value -= int(value);
-  for (unsigned int i=0; i<frac; i++, o += 4) {
-    value *= 10;
-    writeDigit(offset + o, int(value));
-    value -= int(value);
-  }
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::writeFixedPoint(const Offset::Bit &offset, double value, bool sign, unsigned int dec, unsigned int frac) {
-  unsigned int o = 0;
-  writeInteger(offset + o, value, sign, dec);
-  o += 4*dec + (sign ? 4 : 0);
-
-  writeDigit(offset + o, 0xa);
-  o += 4;
-
-  value = std::abs(value);
-  value -= int(value);
-  writeFractional(offset + o, value, false, frac);
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setName(const QString &name) {
-  writeASCII(Offset::name(), name, Limit::nameLength(), 0x00);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setEpoch(const::OrbitalElement::Epoch &epoch) {
-  writeInteger(Offset::epochYear(), epoch.year%100, false, 2);
-  writeFixedPoint(Offset::epochJulienDay(), epoch.toEpoch(), false, 3, 8);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setMeanMotionDerivative(double dmm) {
-  writeFixedPoint(Offset::meanMotionDerivative(), dmm, true, 0, 8);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setInclination(double incl) {
-  writeFixedPoint(Offset::inclination(), incl, false, 3, 4);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setAscension(double asc) {
-  writeFixedPoint(Offset::ascension(), asc, false, 3, 4);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setEccentricity(double ecc) {
-  writeFractional(Offset::eccentricity(), ecc, false, 7);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setPerigee(double arg) {
-  writeFixedPoint(Offset::perigee(), arg, false, 3, 4);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setMeanAnomaly(double ma) {
-  writeFixedPoint(Offset::meanAnomaly(), ma, false, 3, 4);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setMeanMotion(double mm) {
-  writeFixedPoint(Offset::meanMotion(), mm, false, 2, 8);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setRevolutionNumber(unsigned int num) {
-  writeInteger(Offset::revolutionNumber(), num, false, 5);
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setFMDownlink(const Frequency &f) {
-  setUInt32_le(Offset::fmDownlink(), f.inHz()/10);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setFMUplink(const Frequency &f) {
-  setUInt32_le(Offset::fmUplink(), f.inHz()/10);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setCTCSS(const SelectiveCall &call) {
-  if (! call.isCTCSS())
-    return;
-  setUInt32_le(Offset::ctcss(), call.mHz()/100);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setAPRSDownlink(const Frequency &f) {
-  setUInt32_le(Offset::aprsDownlink(), f.inHz()/10);
-}
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setAPRSUplink(const Frequency &f) {
-  setUInt32_le(Offset::aprsUplink(), f.inHz()/10);
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setBeacon(const Frequency &f) {
-  setUInt32_le(Offset::beacon(), f.inHz()/10);
-}
-
-
-void
-OpenGD77BaseCodeplug::OrbitalElement::setAPRSPath(const QString &path) {
-  writeASCII(Offset::aprsPath(), path, Limit::pathLength(), 0x00);
-}
-
-
-bool
-OpenGD77BaseCodeplug::OrbitalElement::encode(const Satellite &sat, const ErrorStack &err) {
-  Q_UNUSED(err);
-
-  // meta
-  setName(sat.name());
-
-  // orbital elements
-  setMeanMotion(sat.meanMotion());
-  setMeanMotionDerivative(sat.meanMotionDerivative());
-  setInclination(sat.inclination());
-  setAscension(sat.ascension());
-  setEccentricity(sat.eccentricity());
-  setPerigee(sat.perigee());
-  setMeanAnomaly(sat.meanAnomaly());
-  setRevolutionNumber(sat.revolutionNumber());
-
-  // transponder
-  setFMDownlink(sat.fmDownlink());
-  setFMUplink(sat.fmUplink());
-  setCTCSS(sat.fmUplinkTone());
-  setAPRSDownlink(sat.aprsDownlink());
-  setAPRSUplink(sat.aprsUplink());
-  setBeacon(sat.beacon());
-
-  /// @bug set APRS path
-
-  return true;
-}
-
-
-
-/* ********************************************************************************************* *
- * Implementation of OpenGD77BaseCodeplug::OrbitalBankElement
- * ********************************************************************************************* */
-OpenGD77BaseCodeplug::OrbitalBankElement::OrbitalBankElement(uint8_t *ptr, size_t size)
-  : Element(ptr, size)
-{
-  // pass...
-}
-
-OpenGD77BaseCodeplug::OrbitalBankElement::OrbitalBankElement(uint8_t *ptr)
-  : Element(ptr, size())
-{
-  // pass...
-}
-
-
-OpenGD77BaseCodeplug::OrbitalElement
-OpenGD77BaseCodeplug::OrbitalBankElement::element(unsigned int i) const {
-  return OrbitalElement(_data + Offset::elements() + i*Offset::betweenElements());
-}
-
-
-bool
-OpenGD77BaseCodeplug::OrbitalBankElement::encode(Context &ctx, const ErrorStack &err) {
-  const SatelliteDatabase *db = ctx.satellites();
-
-  for (unsigned int i=0; i<Limit::elements(); i++) {
-    if (db && (i<db->count())) {
-      const Satellite &sat = db->getAt(i);
-      if (! element(i).encode(sat)) {
-        errMsg(err) << "Cannot encode satellite '" << sat.name() << "' at index " << i << ".";
-        return false;
-      }
-    } else {
-      element(i).clear();
-    }
-  }
-
-  return true;
-}
-
-
-
 /* ********************************************************************************************* *
  * Implementation of OpenGD77BaseCodeplug::ContactElement
  * ********************************************************************************************* */
@@ -2371,6 +2228,352 @@ OpenGD77BaseCodeplug::GroupListBankElement::link(Context &ctx, const ErrorStack
 
 
 
+/* ********************************************************************************************* *
+ * Implementation of OpenGD77BaseCodeplug::SatelliteElement
+ * ********************************************************************************************* */
+OpenGD77BaseCodeplug::SatelliteElement::SatelliteElement(uint8_t *ptr, size_t size)
+  : Element(ptr, size)
+{
+  // pass...
+}
+
+OpenGD77BaseCodeplug::SatelliteElement::SatelliteElement(uint8_t *ptr)
+  : Element(ptr, size())
+{
+  // pass...
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::clear() {
+  memset(_data, 0, size());
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::writeDigit(const Offset::Bit &offset, uint8_t digit) {
+  // Must be bit 0 or 3 (BCD)
+  if (offset.bit % 4)
+    return;
+
+  uint8_t val = getUInt8(offset.byte);
+  val &= ~(0xf << offset.bit);
+  val |= ((digit & 0xf) << offset.bit);
+  setUInt8(offset.byte, val);
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::writeInteger(const Offset::Bit &offset, int value, bool sign, unsigned int dec) {
+  unsigned int o = 0;
+  // Must be bit 0 or 4 (BCD)
+  if (offset.bit % 4)
+    return;
+  if (0 == dec)
+    return;
+
+  if (sign && 0 > value)
+    writeDigit(offset + o, 0xc); // '-' ?!?
+  else
+    writeDigit(offset + o, 0xb); // blank
+
+  o += 4*(dec-1);
+  for (int i=dec; i>0; i--, o = o - 4) {
+    writeDigit(offset + o, value % 10);
+    value /= 10;
+  }
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::writeFractional(const Offset::Bit &offset, double value, bool sign, unsigned int frac) {
+  unsigned int o = 0;
+  if (offset.bit % 4)
+    return;
+
+  if (0 == frac)
+    return;
+
+  if (sign) {
+    if (0 > value)
+      writeDigit(offset + o, 0xc);
+    else
+      writeDigit(offset + o, 0xb);
+    o += 4;
+  }
+
+  value -= int(value);
+  for (unsigned int i=0; i<frac; i++, o += 4) {
+    value *= 10;
+    writeDigit(offset + o, int(value));
+    value -= int(value);
+  }
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::writeFixedPoint(const Offset::Bit &offset, double value, bool sign, unsigned int dec, unsigned int frac) {
+  unsigned int o = 0;
+  writeInteger(offset + o, value, sign, dec);
+  o += 4*dec + (sign ? 4 : 0);
+
+  writeDigit(offset + o, 0xa);
+  o += 4;
+
+  value = std::abs(value);
+  value -= int(value);
+  writeFractional(offset + o, value, false, frac);
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setName(const QString &name) {
+  writeASCII(Offset::name(), name, Limit::nameLength(), 0x00);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setEpoch(const OrbitalElement::Epoch &epoch) {
+  writeInteger(Offset::epochYear(), epoch.year%100, false, 2);
+  writeFixedPoint(Offset::epochJulienDay(), epoch.toEpoch(), false, 3, 8);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setMeanMotionDerivative(double dmm) {
+  writeFixedPoint(Offset::meanMotionDerivative(), dmm, true, 0, 8);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setInclination(double incl) {
+  writeFixedPoint(Offset::inclination(), incl, false, 3, 4);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setAscension(double asc) {
+  writeFixedPoint(Offset::ascension(), asc, false, 3, 4);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setEccentricity(double ecc) {
+  writeFractional(Offset::eccentricity(), ecc, false, 7);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setPerigee(double arg) {
+  writeFixedPoint(Offset::perigee(), arg, false, 3, 4);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setMeanAnomaly(double ma) {
+  writeFixedPoint(Offset::meanAnomaly(), ma, false, 3, 4);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setMeanMotion(double mm) {
+  writeFixedPoint(Offset::meanMotion(), mm, false, 2, 8);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setRevolutionNumber(unsigned int num) {
+  writeInteger(Offset::revolutionNumber(), num, false, 5);
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setFMDownlink(const Frequency &f) {
+  setUInt32_le(Offset::fmDownlink(), f.inHz());
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setFMUplink(const Frequency &f) {
+  setUInt32_le(Offset::fmUplink(), f.inHz());
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setCTCSS(const SelectiveCall &call) {
+  if (! call.isCTCSS())
+    return;
+  setUInt32_le(Offset::ctcss(), call.mHz()/100);
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setAPRSDownlink(const Frequency &f) {
+  setUInt32_le(Offset::aprsDownlink(), f.inHz());
+}
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setAPRSUplink(const Frequency &f) {
+  setUInt32_le(Offset::aprsUplink(), f.inHz());
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setBeacon(const Frequency &f) {
+  setUInt32_le(Offset::beacon(), f.inHz());
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteElement::setAPRSPath(const QString &path) {
+  writeASCII(Offset::aprsPath(), path, Limit::pathLength(), 0x00);
+}
+
+
+bool
+OpenGD77BaseCodeplug::SatelliteElement::encode(const Satellite &sat, const ErrorStack &err) {
+  Q_UNUSED(err);
+
+  // meta
+  setName(sat.name());
+
+  // orbital elements
+  setEpoch(sat.epoch());
+  setMeanMotion(sat.meanMotion());
+  setMeanMotionDerivative(sat.meanMotionDerivative());
+  setInclination(sat.inclination());
+  setAscension(sat.ascension());
+  setEccentricity(sat.eccentricity());
+  setPerigee(sat.perigee());
+  setMeanAnomaly(sat.meanAnomaly());
+  setRevolutionNumber(sat.revolutionNumber());
+
+  // transponder
+  setFMDownlink(sat.fmDownlink());
+  setFMUplink(sat.fmUplink());
+  setCTCSS(sat.fmUplinkTone());
+  setAPRSDownlink(sat.aprsDownlink());
+  setAPRSUplink(sat.aprsUplink());
+  setBeacon(sat.beacon());
+
+  /// @bug set APRS path
+
+  return true;
+}
+
+
+
+/* ********************************************************************************************* *
+ * Implementation of OpenGD77BaseCodeplug::SatelliteBankElement
+ * ********************************************************************************************* */
+OpenGD77BaseCodeplug::SatelliteBankElement::SatelliteBankElement(uint8_t *ptr, size_t size)
+  : Element(ptr, size)
+{
+  // pass...
+}
+
+OpenGD77BaseCodeplug::SatelliteBankElement::SatelliteBankElement(uint8_t *ptr)
+  : Element(ptr, size())
+{
+  // pass...
+}
+
+
+void
+OpenGD77BaseCodeplug::SatelliteBankElement::clear() {
+  memset(_data, 0, size());
+  setUInt32_le(Offset::blockId(), 3);
+  setUInt32_le(Offset::segmentSize(), size()-8);
+}
+
+
+OpenGD77BaseCodeplug::SatelliteElement
+OpenGD77BaseCodeplug::SatelliteBankElement::satellite(unsigned int idx) {
+  return SatelliteElement(_data + Offset::satellites() + idx*Offset::betweenSatellites());
+}
+
+bool
+OpenGD77BaseCodeplug::SatelliteBankElement::encode(SatelliteDatabase *db, const ErrorStack &err) {
+  clear();
+
+  for (unsigned int i=0; i<Limit::satellites(); i++) {
+    SatelliteElement el = satellite(i);
+    el.clear();
+    if (db->count() <= i)
+      continue;
+    if (! el.encode(db->getAt(i), err)) {
+      errMsg(err) << "Cannot encode satellite '" << db->getAt(i).name()
+                  << "' at index " << i << ".";
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
+
+/* ********************************************************************************************* *
+ * Implementation of OpenGD77BaseCodeplug::AdditionalSettingsElement
+ * ********************************************************************************************* */
+OpenGD77BaseCodeplug::AdditionalSettingsElement::AdditionalSettingsElement(uint8_t *ptr, size_t size)
+  : Element(ptr, size)
+{
+  // pass...
+}
+
+OpenGD77BaseCodeplug::AdditionalSettingsElement::AdditionalSettingsElement(uint8_t *ptr)
+  : Element(ptr, size())
+{
+  // pass...
+}
+
+bool
+OpenGD77BaseCodeplug::AdditionalSettingsElement::isValid() const {
+  return Element::isValid() && ("OpenGD77" ==  magic()) && (1 == version());
+}
+
+void
+OpenGD77BaseCodeplug::AdditionalSettingsElement::clear() {
+  memset(_data, 0xff, size());
+  writeASCII(Offset::magicString(), "OpenGD77", Limit::magicStringLength(), 0xff);
+  setUInt32_le(Offset::versionNumber(), 1);
+}
+
+QString
+OpenGD77BaseCodeplug::AdditionalSettingsElement::magic() const {
+  return readASCII(Offset::magicString(), Limit::magicStringLength(), 0xff);
+}
+
+unsigned int
+OpenGD77BaseCodeplug::AdditionalSettingsElement::version() const {
+  return getUInt32_le(Offset::versionNumber());
+}
+
+bool
+OpenGD77BaseCodeplug::AdditionalSettingsElement::hasSettings(Settings set) const {
+  if (! isValid())
+    return false;
+
+  for (unsigned int offset = Offset::blocks(); offset < size(); ) {
+    uint32_t magic = getUInt32_le(offset),
+        content_size = getUInt32_le(offset+4);
+    if (magic == (uint32_t)set)
+      return true;
+    if (0xffffffff == magic)
+      return false;
+    offset += 8 + content_size;
+  }
+
+  return false;
+}
+
+OpenGD77BaseCodeplug::SatelliteBankElement
+OpenGD77BaseCodeplug::AdditionalSettingsElement::satellites() const {
+  if (! isValid())
+    return SatelliteBankElement(nullptr);
+
+  for (unsigned int offset = Offset::blocks(); offset < size(); ) {
+    uint32_t magic = getUInt32_le(offset),
+        content_size = getUInt32_le(offset+4);
+    if ((0xffffffff == magic) || (magic == (uint32_t)Settings::SatelliteOrbitals))
+      return SatelliteBankElement(_data + offset);
+    offset += 8 + content_size;
+  }
+
+  return SatelliteBankElement(nullptr);
+}
+
+
+
 /* ********************************************************************************************* *
  * Implementation of OpenGD77BaseCodeplug
  * ********************************************************************************************* */
@@ -2621,6 +2824,11 @@ OpenGD77BaseCodeplug::decodeElements(Context &ctx, const ErrorStack &err) {
     return false;
   }
 
+  if (! this->linkAPRSSettings(ctx, err)) {
+    errMsg(err) << "Cannot decode APRS settings.";
+    return false;
+  }
+
   return true;
 }
 
