@@ -18,7 +18,7 @@ OpenUV380Codeplug::OpenUV380Codeplug(QObject *parent)
   addImage("OpenGD77 Codeplug FLASH");
   image(FLASH).addElement(0x00000080, 0x00005fe0);
   image(FLASH).addElement(0x00007500, 0x00003b00);
-  image(FLASH).addElement(0x00020000, 0x000011a0);
+  image(FLASH).addElement(0x00020000, AdditionalSettingsElement::size());
   image(FLASH).addElement(0x0009b000, 0x00013e60);
 }
 
@@ -30,7 +30,7 @@ OpenUV380Codeplug::clearGeneralSettings() {
 bool
 OpenUV380Codeplug::encodeGeneralSettings(const Flags &flags, Context &ctx, const ErrorStack &err) {
   GeneralSettingsElement el(data(Offset::settings(), ImageIndex::settings()));
-  if (! flags.updateCodePlug)
+  if (! flags.updateCodeplug())
     el.clear();
 
   return el.encode(ctx, err);
@@ -68,7 +68,7 @@ OpenUV380Codeplug::clearAPRSSettings() {
 bool
 OpenUV380Codeplug::encodeAPRSSettings(const Flags &flags, Context &ctx, const ErrorStack &err) {
   APRSSettingsBankElement el(data(Offset::aprsSettings(), ImageIndex::aprsSettings()));
-  if (! flags.updateCodePlug)
+  if (! flags.updateCodeplug())
     el.clear();
 
   return el.encode(ctx, err);
