@@ -1,15 +1,13 @@
 #include "radiolimits.hh"
 #include "configobject.hh"
-#include "logger.hh"
 #include "config.hh"
 #include <QMetaProperty>
 #include <QRegularExpression>
-#include <ctype.h>
 
 // Utility function to check string content for ASCII encoding
 inline bool qstring_is_ascii(const QString &text) {
   foreach (QChar c, text) {
-    if ((c<0x1f) && (0x7f != c))
+    if ((c.unicode() < 0x1f) && (0x7f != c.unicode()))
       return false;
   }
   return true;
@@ -17,7 +15,7 @@ inline bool qstring_is_ascii(const QString &text) {
 
 // Utility function to check string content for DTMF encoding
 inline bool qstring_is_dtmf(const QString &text) {
-  return QRegularExpression("^[0-9A-Da-d*#]*$").match(text).isValid();
+  return QRegularExpression("^[0-9A-Da-d*#]*$").match(text).hasMatch();
 }
 
 
@@ -62,6 +60,11 @@ RadioLimitIssue::message() const {
   return _message;
 }
 
+const QStringList &
+RadioLimitIssue::stack() const {
+  return _stack;
+}
+
 QString
 RadioLimitIssue::format() const {
   QString res; QTextStream stream(&res);
@@ -71,7 +74,7 @@ RadioLimitIssue::format() const {
   case Warning: stream << "Warn: "; break;
   case Critical: stream << "Crit: "; break;
   }
-  stream << "In " << _stack.join(", ") << ": " << _message;
+  stream << _stack.join(", ") << ": " << _message;
   stream.flush();
   return res;
 }
@@ -154,7 +157,7 @@ RadioLimitIgnored::RadioLimitIgnored(RadioLimitIssue::Severity notify, QObject *
 
 bool
 RadioLimitIgnored::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  ConfigObject *obj = prop.read(item).value<ConfigObject *>();
+  auto obj = prop.read(item).value<const ConfigObject *>();
   if (nullptr != obj)
     return verifyObject(obj, context);
 
@@ -175,8 +178,8 @@ RadioLimitIgnored::verifyObject(const ConfigObject *item, RadioLimitContext &con
 /* ********************************************************************************************* *
  * Implementation of RadioLimitValue
  * ********************************************************************************************* */
-RadioLimitValue::RadioLimitValue(QObject *parent)
-  : RadioLimitElement(parent)
+RadioLimitValue::RadioLimitValue(RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitElement(parent), _severity(severity)
 {
   // pass...
 }
@@ -185,71 +188,80 @@ RadioLimitValue::RadioLimitValue(QObject *parent)
 /* ********************************************************************************************* *
  * Implementation of RadioLimitString
  * ********************************************************************************************* */
-RadioLimitString::RadioLimitString(int minLen, int maxLen, Encoding enc, QObject *parent)
-  : RadioLimitValue(parent), _minLen(minLen), _maxLen(maxLen), _encoding(enc)
+RadioLimitString::RadioLimitString(int minLen, int maxLen, Encoding enc, RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _minLen(minLen), _maxLen(maxLen), _encoding(enc)
 {
   // pass...
 }
 
 bool
 RadioLimitString::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  if (QVariant::String != prop.type()) {
+  if (QMetaType::QString != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "Cannot check property " << prop.name() << ": Expected string.";
     return false;
   }
 
   QString value = prop.read(item).toString();
+  bool success = true;
 
   if ((0<_maxLen) && (value.size() > int(_maxLen))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "String length of " << prop.name() << " ('" << value << "', " << value.size()
         << ") exceeds maximum length " << _maxLen << ".";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
   if ((0<_minLen) && (value.size() < int(_minLen))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "String length of " << prop.name() << " ('" << value << "', " << value.size()
         << ") is shorter than minimum size " << _minLen << ".";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
   if ((ASCII == _encoding) && (! qstring_is_ascii(value))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "Cannot encode string '" << value << "' in ASCII.";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   } else if ((DTMF == _encoding) && (! qstring_is_dtmf(value))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "Cannot encode string '" << value << "' in DTMF.";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
 /* ********************************************************************************************* *
  * Implementation of RadioLimitStringRegEx
  * ********************************************************************************************* */
-RadioLimitStringRegEx::RadioLimitStringRegEx(const QString &pattern, QObject *parent)
-  : RadioLimitValue(parent), _pattern(pattern)
+RadioLimitStringRegEx::RadioLimitStringRegEx(const QString &pattern, RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _pattern(pattern)
 {
   // pass...
 }
 
 bool
 RadioLimitStringRegEx::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  if (QVariant::String != prop.type()) {
+  if (QMetaType::QString != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "Cannot check property " << prop.name() << ": Expected string.";
     return false;
   }
 
+  bool success = true;
+
   QString value = prop.read(item).toString();
-  if (! _pattern.exactMatch(value)) {
-    auto &msg = context.newMessage(RadioLimitIssue::Warning);
+  auto match = _pattern.match(value);
+  if (! match.hasMatch()) {
+    auto &msg = context.newMessage(_severity);
     msg << "Value '" << value << "' of property " << prop.name()
         << " does not match pattern '" << _pattern.pattern() << "'.";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
@@ -257,26 +269,29 @@ RadioLimitStringRegEx::verify(const ConfigItem *item, const QMetaProperty &prop,
  * Implementation of RadioLimitStringIgnored
  * ********************************************************************************************* */
 RadioLimitStringIgnored::RadioLimitStringIgnored(RadioLimitIssue::Severity severity, QObject *parent)
-  : RadioLimitValue(parent), _severity(severity)
+  : RadioLimitValue(severity, parent)
 {
   // pass...
 }
 
 bool
 RadioLimitStringIgnored::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  if (QVariant::String != prop.type()) {
+  if (QMetaType::QString != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Warning);
     msg = tr("Expected value of '%1' to be string.").arg(prop.name());
     return true;
   }
 
+  bool success = true;
+
   QVariant value = prop.read(item);
   if (! value.toString().isEmpty()) {
     auto &msg = context.newMessage(_severity);
     msg = tr("Value of '%1' is ignored. Not applicable/supported by the radio.").arg(prop.name());
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
@@ -284,7 +299,7 @@ RadioLimitStringIgnored::verify(const ConfigItem *item, const QMetaProperty &pro
  * Implementation of RadioLimitBool
  * ********************************************************************************************* */
 RadioLimitBool::RadioLimitBool(QObject *parent)
-  : RadioLimitValue(parent)
+  : RadioLimitValue(RadioLimitIssue::Severity::Hint, parent)
 {
   // pass...
 }
@@ -293,7 +308,7 @@ bool
 RadioLimitBool::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
   Q_UNUSED(item)
 
-  if (QVariant::Bool != prop.type()) {
+  if (QMetaType::Bool != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "Cannot check property " << prop.name() << ": Expected bool.";
     return false;
@@ -314,60 +329,66 @@ RadioLimitIgnoredBool::RadioLimitIgnoredBool(RadioLimitIssue::Severity notify, Q
 
 bool
 RadioLimitIgnoredBool::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  if (QVariant::Bool != prop.type()) {
+  if (QMetaType::Bool != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "Cannot check property " << prop.name() << ": Expected bool.";
     return false;
   }
 
+  bool success = true;
   bool value = prop.read(item).toBool();
   if (value) {
     auto &msg = context.newMessage(_severity);
     msg << "Setting " << prop.name() << " is ignored.";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
 /* ********************************************************************************************* *
  * Implementation of RadioLimitUInt
  * ********************************************************************************************* */
-RadioLimitUInt::RadioLimitUInt(qint64 minValue, qint64 maxValue, qint64 defValue, QObject *parent)
-  : RadioLimitValue(parent), _minValue(minValue), _maxValue(maxValue), _defValue(defValue)
+RadioLimitUInt::RadioLimitUInt(qint64 minValue, qint64 maxValue, qint64 defValue,
+                               RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _minValue(minValue), _maxValue(maxValue), _defValue(defValue)
 {
   // pass...
 }
 
 bool
 RadioLimitUInt::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
-  if (QVariant::UInt != prop.type()) {
+  if (QMetaType::UInt != prop.typeId()) {
     auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "Cannot check property " << prop.name() << ": Expected uint.";
     return false;
   }
 
+  bool success = true;
   unsigned value = prop.read(item).toUInt();
 
   if ((0<_maxValue) && (value > qint64(_maxValue)) && ((0>_defValue) || (value!=_defValue))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "Value " << value << " of " << prop.name() << " exceeds maximum " << _maxValue << ".";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
   if ((0<_minValue) && (value < qint64(_minValue)) && ((0>_defValue) || (value!=_defValue))) {
-    auto &msg = context.newMessage();
+    auto &msg = context.newMessage(_severity);
     msg << "Value " << value << " of " << prop.name() << " is smaller than minimum " << _minValue << ".";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
 /* ********************************************************************************************* *
  * Implementation of RadioLimitDMRId
  * ********************************************************************************************* */
-RadioLimitDMRId::RadioLimitDMRId(QObject *parent)
-  : RadioLimitUInt(1, 16777215, -1, parent)
+RadioLimitDMRId::RadioLimitDMRId(RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitUInt(1, 16777215, -1, severity, parent)
 {
   // pass...
 }
@@ -376,8 +397,8 @@ RadioLimitDMRId::RadioLimitDMRId(QObject *parent)
 /* ********************************************************************************************* *
  * Implementation of RadioLimitEnum
  * ********************************************************************************************* */
-RadioLimitEnum::RadioLimitEnum(const std::initializer_list<unsigned> &values, QObject *parent)
-  : RadioLimitValue(parent), _values(values)
+RadioLimitEnum::RadioLimitEnum(const std::initializer_list<unsigned> &values, RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _values(values)
 {
   // pass...
 }
@@ -390,7 +411,9 @@ RadioLimitEnum::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
     return false;
   }
 
+  bool success = true;
   unsigned value = prop.read(item).toUInt();
+
   if (! _values.contains(value)) {
     QMetaEnum e = prop.enumerator();
     QStringList possible;
@@ -401,23 +424,24 @@ RadioLimitEnum::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
     msg << "The enum value '" << e.valueToKey(value) << "' cannot be encoded. "
         << "Valid values are " << possible.join(", ") << ". "
         << "Another value might be chosen automatically.";
+    success &= (_severity < RadioLimitIssue::Severity::Critical);
   }
 
-  return true;
+  return success;
 }
 
 
 /* ********************************************************************************************* *
  * Implementation of RadioLimitFrequencies
  * ********************************************************************************************* */
-RadioLimitFrequencies::RadioLimitFrequencies(QObject *parent)
-  : RadioLimitValue(parent), _frequencyRanges()
+RadioLimitFrequencies::RadioLimitFrequencies(RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _frequencyRanges()
 {
   // pass...
 }
 
-RadioLimitFrequencies::RadioLimitFrequencies(const RangeList &ranges, bool warnOnly, QObject *parent)
-  : RadioLimitValue(parent), _frequencyRanges(), _warnOnly(warnOnly)
+RadioLimitFrequencies::RadioLimitFrequencies(const RangeList &ranges, RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitValue(severity, parent), _frequencyRanges()
 {
   for (auto range=ranges.begin(); range!=ranges.end(); range++) {
     _frequencyRanges.append({range->first, range->second});
@@ -444,26 +468,24 @@ RadioLimitFrequencies::verify(const ConfigItem *item, const QMetaProperty &prop,
   if (context.ignoreFrequencyLimits())
     return true;
 
-  auto &msg = context.newMessage(RadioLimitIssue::Warning);
+  auto &msg = context.newMessage(_severity);
   msg << "Frequency " << value.inMHz() << "MHz is outside of allowed frequency ranges.";
 
-  if(_warnOnly)
-    return true;
-  return false;
+  return _severity < RadioLimitIssue::Severity::Critical;
 }
 
 
 /* ********************************************************************************************* *
  * Implementation of RadioLimitTransmitFrequencies
  * ********************************************************************************************* */
-RadioLimitTransmitFrequencies::RadioLimitTransmitFrequencies(QObject *parent)
-  : RadioLimitFrequencies(parent)
+RadioLimitTransmitFrequencies::RadioLimitTransmitFrequencies(RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitFrequencies(severity, parent)
 {
   // pass...
 }
 
-RadioLimitTransmitFrequencies::RadioLimitTransmitFrequencies(const RangeList &ranges, QObject *parent)
-  : RadioLimitFrequencies(ranges, false, parent)
+RadioLimitTransmitFrequencies::RadioLimitTransmitFrequencies(const RangeList &ranges, RadioLimitIssue::Severity severity, QObject *parent)
+  : RadioLimitFrequencies(ranges, severity, parent)
 {
   // pass...
 }
@@ -527,7 +549,7 @@ RadioLimitItem::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
   if (prop.read(item).isNull())
     return true;
 
-  context.push(QString("Property '%1'").arg(prop.name()));
+  context.push(QString("In property '%1'").arg(prop.name()));
   bool success = verifyItem(prop.read(item).value<ConfigItem*>(), context);
   context.pop();
   return success;
@@ -570,9 +592,9 @@ RadioLimitObject::RadioLimitObject(const PropList &list, QObject *parent)
 
 bool
 RadioLimitObject::verifyObject(const ConfigObject *item, RadioLimitContext &context) const {
-  //context.push(QString("Object '%1'").arg(item->name()));
+  context.push(QString("In object '%1'").arg(item->name()));
   bool success = verifyItem(item, context);
-  //context.pop();
+  context.pop();
   return success;
 }
 
@@ -610,6 +632,13 @@ RadioLimitObjRef::RadioLimitObjRef(const QMetaObject &type, bool allowNull, QObj
   _types.insert(type.className());
 }
 
+RadioLimitObjRef::RadioLimitObjRef(const MetaObjectList &types, bool allowNull, QObject *parent)
+  : RadioLimitElement(parent), _allowNull(allowNull), _types()
+{
+  for (auto type=types.begin(); type!=types.end(); type++)
+    _types.insert(type->className());
+}
+
 bool
 RadioLimitObjRef::verify(const ConfigItem *item, const QMetaProperty &prop, RadioLimitContext &context) const {
   ConfigObjectReference *ref = prop.read(item).value<ConfigObjectReference *>();
@@ -719,7 +748,7 @@ RadioLimitList::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
   foreach (QString type, _elements.keys())
     counts.insert(type,0);
 
-  context.push(QString("List '%1'").arg(prop.name()));
+  context.push(QString("In list '%1'").arg(prop.name()));
 
   // Check type and structure
   for (int i=0; i<plist->count(); i++) {
@@ -736,7 +765,7 @@ RadioLimitList::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
 
     counts[className]++;
 
-    context.push(QString("Element %1 ('%2')").arg(i).arg(obj->name()));
+    context.push(QString("In element %1 ('%2')").arg(i).arg(obj->name()));
     if (! _elements[className]->verifyObject(obj, context)) {
       context.pop();
       context.pop();
@@ -748,7 +777,7 @@ RadioLimitList::verify(const ConfigItem *item, const QMetaProperty &prop, RadioL
   // Check counts
   foreach (QString className, _elements.keys()) {
     if ((0 <= _minCount[className]) && (counts[className]<_minCount[className])) {
-      auto &msg = context.newMessage(RadioLimitIssue::Warning);
+      auto &msg = context.newMessage(RadioLimitIssue::Critical);
       msg << "The number of elements of type '" << className << "' " << counts[className]
              << " is less than the required count " << _minCount[className] << ".";
     }
@@ -799,7 +828,7 @@ RadioLimitRefList::verify(const ConfigItem *item, const QMetaProperty &prop, Rad
 
   const ConfigObjectRefList *plist = prop.read(item).value<ConfigObjectRefList*>();
   if ((0 <= _minSize) && (_minSize > plist->count())) {
-    auto &msg = context.newMessage(RadioLimitIssue::Warning);
+    auto &msg = context.newMessage(RadioLimitIssue::Critical);
     msg << "List '" << prop.name() << "' requires at least " << _minSize
         << " elements, " << plist->count() << " elements found.";
     return false;
@@ -921,17 +950,22 @@ RadioLimitSingleZone::verifyItem(const ConfigItem *item, RadioLimitContext &cont
  * Implementation of RadioLimits
  * ********************************************************************************************* */
 RadioLimits::RadioLimits(bool betaWarning, QObject *parent)
-  : RadioLimitItem(parent), _betaWarning(betaWarning)
+  : RadioLimitItem(parent), _betaWarning(betaWarning),
+    _hasCallSignDB(false), _callSignDBImplemented(false), _numCallSignDBEntries(0),
+    _hasSatelliteConfig(false), _satelliteConfigImplemented(false), _numSatellites(0)
 {
   // pass...
 }
 
 RadioLimits::RadioLimits(const std::initializer_list<std::pair<QString, RadioLimitElement *> > &list, QObject *parent)
-  : RadioLimitItem(list, parent)
+  : RadioLimitItem(list, parent),
+    _hasCallSignDB(false), _callSignDBImplemented(false), _numCallSignDBEntries(0),
+    _hasSatelliteConfig(false), _satelliteConfigImplemented(false), _numSatellites(0)
 {
   // pass...
 }
 
+
 bool
 RadioLimits::hasCallSignDB() const {
   return _hasCallSignDB;
@@ -947,6 +981,23 @@ RadioLimits::numCallSignDBEntries() const {
   return _numCallSignDBEntries;
 }
 
+
+bool
+RadioLimits::hasSatelliteConfig() const {
+  return _hasSatelliteConfig;
+}
+
+bool
+RadioLimits::satelliteConfigImplemented() const {
+  return _satelliteConfigImplemented;
+}
+
+unsigned
+RadioLimits::numSatellites() const {
+  return _numSatellites;
+}
+
+
 bool
 RadioLimits::verifyConfig(const Config *config, RadioLimitContext &context) const {
   if (_betaWarning) {
