@@ -89,6 +89,12 @@ Visitor::processProperty(ConfigItem *item, const QMetaProperty &prop, const Erro
                   << item->metaObject()->className() << "'.";
       return false;
     }
+  } else if (QString("SelectiveCall") == prop.typeName()) {
+    if (! this->processSelectiveCall(item, prop, err)) {
+      errMsg(err) << "While processing frequency '" << prop.name() << "' of '"
+                  << item->metaObject()->className() << "'.";
+      return false;
+    }
   } else if (ConfigObjectReference *ref = prop.read(item).value<ConfigObjectReference *>()) {
     if (! this->processReference(ref, err)) {
       errMsg(err) << "While processing reference '" << prop.name() << "' of '"
@@ -186,6 +192,13 @@ Visitor::processInterval(ConfigItem *parent, const QMetaProperty &prop, const Er
   return true;
 }
 
+bool
+Visitor::processSelectiveCall(ConfigItem *parent, const QMetaProperty &prop, const ErrorStack &err) {
+  Q_UNUSED(parent); Q_UNUSED(prop); Q_UNUSED(err)
+  // Does nothing, return true;
+  return true;
+}
+
 bool
 Visitor::processUnknownType(ConfigItem *parent, const QMetaProperty &prop, const ErrorStack &err) {
   errMsg(err) << "Cannot handle property '" << prop.name() << "' of '"
