@@ -36,7 +36,7 @@ ChannelListView::~ChannelListView() {
 
 void
 ChannelListView::onAddAnalogChannel() {
-  AnalogChannelDialog dialog(_config);
+  AnalogChannelDialog dialog(_config, parentWidget());
   if (QDialog::Accepted != dialog.exec())
     return;
 
@@ -48,7 +48,7 @@ ChannelListView::onAddAnalogChannel() {
 
 void
 ChannelListView::onAddDigitalChannel() {
-  DigitalChannelDialog dialog(_config);
+  DigitalChannelDialog dialog(_config, parentWidget());
   if (QDialog::Accepted != dialog.exec())
     return;
 
@@ -77,7 +77,7 @@ ChannelListView::onCloneChannel() {
   // Dispatch by type
   if (channel->is<FMChannel>()) {
     // clone channel
-    FMChannel *clone = new FMChannel(*(channel->as<FMChannel>()));
+    FMChannel *clone = channel->clone()->as<FMChannel>();
     // open editor
     AnalogChannelDialog dialog(_config, clone);
     if (QDialog::Accepted != dialog.exec()) {
@@ -91,7 +91,7 @@ ChannelListView::onCloneChannel() {
     _config->channelList()->add(clone, row+1);
   } else {
     // clone channel
-    DMRChannel *clone = new DMRChannel(*(channel->as<DMRChannel>()));
+    DMRChannel *clone = channel->clone()->as<DMRChannel>();
     // open editor
     DigitalChannelDialog dialog(_config, clone);
     if (QDialog::Accepted != dialog.exec()) {
