@@ -126,6 +126,15 @@ Settings::setUpdateCodeplug(bool update) {
   setValue("updateCodeplug", update);
 }
 
+bool
+Settings::updateDeviceClock() const {
+  return value("updateDeviceClock", false).toBool();
+}
+void
+Settings::setUpdateDeviceClock(bool update) {
+  setValue("updateDeviceClock", update);
+}
+
 bool
 Settings::autoEnableGPS() const {
   return value("autoEnableGPS", false).toBool();
@@ -156,9 +165,10 @@ Settings::setLastDirectoryDir(const QDir &dir) {
 Codeplug::Flags
 Settings::codePlugFlags() const {
   Codeplug::Flags flags;
-  flags.updateCodePlug = updateCodeplug();
-  flags.autoEnableGPS  = autoEnableGPS();
-  flags.autoEnableRoaming = autoEnableRoaming();
+  flags.setUpdateDeviceClock(updateDeviceClock());
+  flags.setUpdateCodeplug(updateCodeplug());
+  flags.setAutoEnableGPS(autoEnableGPS());
+  flags.setAutoEnableRoaming(autoEnableRoaming());
   return flags;
 }
 
@@ -403,6 +413,7 @@ SettingsDialog::SettingsDialog(QWidget *parent)
 
   Ui::SettingsDialog::disableAutoDetect->setChecked(settings.disableAutoDetect());
   Ui::SettingsDialog::updateCodeplug->setChecked(settings.updateCodeplug());
+  Ui::SettingsDialog::updateDeviceClock->setChecked(settings.updateDeviceClock());
   Ui::SettingsDialog::autoEnableGPS->setChecked(settings.autoEnableGPS());
   Ui::SettingsDialog::autoEnableRoaming->setChecked(settings.autoEnableRoaming());
   Ui::SettingsDialog::ignoreVerificationWarnings->setChecked(settings.ignoreVerificationWarning());
@@ -492,6 +503,7 @@ SettingsDialog::accept() {
   settings.enableRadioIdRepeaterSource(Ui::SettingsDialog::radioIdEnable->isChecked());
 
   settings.setUpdateCodeplug(updateCodeplug->isChecked());
+  settings.setUpdateDeviceClock(updateDeviceClock->isChecked());
   settings.setAutoEnableGPS(autoEnableGPS->isChecked());
   settings.setAutoEnableRoaming(autoEnableRoaming->isChecked());
   settings.setIgnoreVerificationWarning(ignoreVerificationWarnings->isChecked());
