@@ -6,6 +6,7 @@
 #include <QTest>
 #include "logger.hh"
 #include <iostream>
+#include "gd73_limits.hh"
 
 #include "configcopyvisitor.hh"
 
@@ -170,5 +171,28 @@ ConfigTest::testCTCSSNull() {
 }
 
 
+void
+ConfigTest::testDMRIdVerification() {
+  GD73Limits limits;
+  RadioLimitContext ctx;
+
+  ErrorStack err;
+  Config cfg;
+
+  if (! cfg.readYAML(":/data/config_test.yaml", err)) {
+    QFAIL(QString("Cannot open codeplug file: %1")
+          .arg(err.format()).toStdString().c_str());
+  }
+
+  QVERIFY(limits.verifyConfig(&cfg, ctx));
+  QVERIFY(ctx.maxSeverity() < RadioLimitIssue::Severity::Critical);
+
+  cfg.radioIDs()->getId(0)->setNumber(0x12345678);
+
+  QVERIFY(!limits.verifyConfig(&cfg, ctx));
+  QVERIFY(ctx.maxSeverity() == RadioLimitIssue::Severity::Critical);
+}
+
+
 QTEST_GUILESS_MAIN(ConfigTest)
 
